<!doctype html>
<html lang="en">
<head>
  <meta charset="utf-8">
  <script src="{{static_url('bundle.js')}}" type="text/javascript" charset="utf-8"></script>
  <script type="text/javascript">
    /* Remove token from URL. */
    (function () {
      var parsedUrl = new URL(window.location.href);
      if (parsedUrl.searchParams.get('token')) {
        parsedUrl.searchParams.delete('token');
        window.history.replaceState({ }, '', parsedUrl.href);
      }
    })();
  </script>
</head>
<body>
  <div id="mydiv">
  <h1>Hello world! </h1>
  </div>
</body>
